package service

import (
	"encoding/json"
	"errors"
	"net/url"

	"gitee.com/yansen_zh/wechat-oplatform-sdk/api"
	"gitee.com/yansen_zh/wechat-oplatform-sdk/entity"
	"gitee.com/yansen_zh/wechat-oplatform-sdk/utils"
	"gitee.com/yansen_zh/wechat-oplatform-sdk/utils/request"
)

// AuthorizerInfoParam 获取授权帐号信息参数
type AuthorizerInfoParam struct {
	ComponentAppId  string `json:"component_appid"`
	AuthorizerAppId string `json:"authorizer_appid"`
}

// AuthorizerInfoResult 获取授权帐号信息结果
type AuthorizerInfoResult struct {
	entity.Error
	AuthorizationInfo entity.AuthorizationInfo `json:"authorization_info"`
	AuthorizerInfo    entity.AuthorizerInfo    `json:"authorizer_info"`
}

// GetAuthorizerInfo 获取授权帐号信息
func GetAuthorizerInfo(componentAccessToken string, param AuthorizerInfoParam) (*AuthorizerInfoResult, error) {
	if componentAccessToken == "" {
		return nil, errors.New("获取授权帐号信息 第三方平台的 component_access_token 不能为空")
	}

	if param.ComponentAppId == "" {
		return nil, errors.New("获取授权帐号信息 第三方平台的 appid 不能为空")
	}

	if param.AuthorizerAppId == "" {
		return nil, errors.New("获取授权帐号信息 授权方 appid 不能为空")
	}

	data, e1 := json.Marshal(param)
	if e1 != nil {
		return nil, e1
	}

	queryParam := url.Values{}
	queryParam.Set("component_access_token", componentAccessToken)

	apiUrl, _ := utils.ParseURL(api.GetAuthorizerInfo, &queryParam)
	resp, e2 := request.PostJSON(apiUrl.String(), data)
	if e2 != nil {
		return nil, e2
	}

	result := AuthorizerInfoResult{}
	if err := json.Unmarshal(resp, &result); err != nil {
		return nil, err
	}

	if result.Code != 0 {
		return &result, result.Error
	}

	return &result, nil
}
