package service

import (
	"encoding/json"
	"errors"

	"gitee.com/yansen_zh/wechat-oplatform-sdk/api"
	"gitee.com/yansen_zh/wechat-oplatform-sdk/entity"
	"gitee.com/yansen_zh/wechat-oplatform-sdk/utils/request"
)

// ComponentTokenParam 获取令牌参数
type ComponentTokenParam struct {
	ComponentAppId        string `json:"component_appid"`
	ComponentAppSecret    string `json:"component_appsecret"`
	ComponentVerifyTicket string `json:"component_verify_ticket"`
}

// ComponentTokenResult 获取令牌结果
type ComponentTokenResult struct {
	entity.Error
	ComponentAccessToken string  `json:"component_access_token"`
	ExpiresIn            float64 `json:"expires_in"`
}

// ComponentVerifyTicket 获取令牌
func ComponentToken(param ComponentTokenParam) (*ComponentTokenResult, error) {
	if param.ComponentAppId == "" {
		return nil, errors.New("获取令牌 第三方平台的 appid 不能为空")
	}

	if param.ComponentAppSecret == "" {
		return nil, errors.New("获取令牌 第三方平台的 appsecret 不能为空")
	}

	if param.ComponentVerifyTicket == "" {
		return nil, errors.New("获取令牌 推送 ticket 不能为空")
	}

	data, e1 := json.Marshal(param)
	if e1 != nil {
		return nil, e1
	}

	resp, e2 := request.PostJSON(api.ComponentToken, data)
	if e2 != nil {
		return nil, e2
	}

	result := ComponentTokenResult{}
	if err := json.Unmarshal(resp, &result); err != nil {
		return nil, err
	}

	if result.Code != 0 {
		return &result, result.Error
	}

	return &result, nil
}
