package service

import (
	"encoding/json"
	"errors"
	"net/url"

	"gitee.com/yansen_zh/wechat-oplatform-sdk/api"
	"gitee.com/yansen_zh/wechat-oplatform-sdk/entity"
	"gitee.com/yansen_zh/wechat-oplatform-sdk/utils"
	"gitee.com/yansen_zh/wechat-oplatform-sdk/utils/request"
)

// PreAuthCodeParam 获取预授权码参数
type PreAuthCodeParam struct {
	ComponentAppId string `json:"component_appid"`
}

// PreAuthCodeResult 获取预授权码结果
type PreAuthCodeResult struct {
	entity.Error
	PreAuthCode string  `json:"pre_auth_code"`
	ExpiresIn   float64 `json:"expires_in"`
}

// CreatePreAuthCode 获取预授权码
func CreatePreAuthCode(componentAccessToken string, param PreAuthCodeParam) (*PreAuthCodeResult, error) {
	if componentAccessToken == "" {
		return nil, errors.New("获取预授权码 第三方平台的 component_access_token 不能为空")
	}

	if param.ComponentAppId == "" {
		return nil, errors.New("获取预授权码 第三方平台的 appid 不能为空")
	}

	data, e1 := json.Marshal(param)
	if e1 != nil {
		return nil, e1
	}

	queryParam := url.Values{}
	queryParam.Set("component_access_token", componentAccessToken)

	apiUrl, _ := utils.ParseURL(api.CreatePreAuthCode, &queryParam)
	resp, e2 := request.PostJSON(apiUrl.String(), data)
	if e2 != nil {
		return nil, e2
	}

	result := PreAuthCodeResult{}
	if err := json.Unmarshal(resp, &result); err != nil {
		return nil, err
	}

	if result.Code != 0 {
		return &result, result.Error
	}

	return &result, nil
}
